/* module KMLImporter:
 * Methods for the importer of the SketchUp plugin KMLTools.
 */



if (!window.AE) { AE = {}; }



AE.KMLImporter = function(self) {



var $ = AE.$;



// Whether the importer is ready to start right now.
var ready = true;



self.initialize = function() {
  $("#button_close").onclick = function() {
    AE.Dialog.close();
  };
  // Prevent page reloading on buttons in form element.
  $("form")[0].onsubmit = function() { return false };

  AE.Dialog.adjustSize();
};



self.show_report = function(imported, failed, log) {
  AE.Style.hide($("#progress_indicator"));
  // Write report text.
  var txt = "";
  txt += AE.Translate.get("Imported features:") + "\n";
  for (var feature in imported) {
    txt += "  " + feature + Array(Math.abs(15-feature.length)).join(" ") + imported[feature] + "\n";
  }
  txt += "\n";
  txt_failed = "";
  for (var feature in failed) {
    txt_failed += "  " + feature + Array(Math.abs(15-feature.length)).join(" ") + failed[feature] + "\n";
  }
  if (txt_failed.length > 0) {
    txt += AE.Translate.get("Not imported:") + "\n";
    txt += txt_failed;
    txt += "\n";
  }
  for (var i=0; i<log.length; i++) {
    txt += log[i].join("\n  ");
    txt += "\n\n";
  }
  $("#report_text").value = txt;
  AE.Style.show($("#report_panel"));
  document.body.style.minWidth = "32em"; // workaround

  AE.Dialog.adjustSize();
}



self.loadFile = function(string) {
  if (typeof string !== "string") { return; }
  AE.Style.show($("#progress_indicator"));
  AE.Dialog.adjustSize();
  // Wait until ready.
  new AE.Scheduler(1).wait( function() { return ready; }, function() {
    var xml = loadXMLString(string);
    var json = xmlToJson(xml);
    AE.Bridge.callRuby("receive_xml", json);
  });
}



// Private methods



// Load XML
var loadXMLString = function(string) {
  if (string == null) { string = $("#input").value };
  if (window.DOMParser) {
    parser = new DOMParser();
    xmlDoc = parser.parseFromString(string, "text/xml");
  } else { // Internet Explorer
    xmlDoc = new ActiveXObject("Microsoft.XMLDOM");
    xmlDoc.async = false;
    xmlDoc.loadXML(string);
    xmlDoc.setProperty("SelectionLanguage","XPath");
  }
  return xmlDoc.documentElement;
}



// Convert XML to JSON
var xmlToJson = function(xml) {
  // Create the return object
  var obj = {};
  if (xml.nodeType == 1) { // ELEMENT_NODE
    // do attributes
    if (xml.attributes.length > 0) {
    obj["@attributes"] = {};
      for (var j = 0; j < xml.attributes.length; j++) {
        var attribute = xml.attributes.item(j);
        obj["@attributes"][attribute.nodeName] = attribute.nodeValue;
      }
    }
  } else if (xml.nodeType == 3) { // TEXT_NODE
    obj = xml.nodeValue;
  } else if (xml.nodeType == 4) { // CDATA_SECTION_NODE
    obj = xml.nodeValue;
  }
  // do children
  if (xml.hasChildNodes()) {
    for (var i = 0; i < xml.childNodes.length; i++) {
      var item = xml.childNodes.item(i);
      var nodeName = item.nodeName;
      // TEXT_NODE or CDATA_SECTION_NODE // Modification of original algorithm.
      if (item.nodeType == 3 || item.nodeType == 4) {
        return item.nodeValue;
      } else if (typeof obj[nodeName] === "undefined") {
        obj[nodeName] = xmlToJson(item);
      } else { // ELEMENT_NODE
        if (Object.prototype.toString.call(obj[nodeName]) !== '[object Array]') {
          var old = obj[nodeName];
          obj[nodeName] = [];
          obj[nodeName].push(old);
        }
        obj[nodeName].push(xmlToJson(item));
      }
    }
  }
  return obj;
};



var getJson = function(url) {
  var s = document.createElement("script");
  s.setAttribute("src", url);
  document.body.appendChild(s);
  void(0);
};



// Private methods



// Function to get elevation data from a web service in case it is not contained in the model.
self.getElevation = function(lat, lon) {
  // Lock the importer to wait until the elevation is returned.
  ready = false;
  // Fallback if json request is unsuccessful.
  window.setTimeout(function(){ ready = true; }, 5000); // TODO: No need to wait if Sketchup.is_online? == false
  // Request json data from geonames.org astergdem API.
  var callback = "AE.KMLImporter.receiveElevation";
  var url = "http://api.geonames.org/astergdemJSON?formatted=true&lat="
  url += lat + "&lng=" + lon + "&username=aerilius&style=full&callback=" + callback;
  getJson(url);
};



self.receiveElevation = function(json) {
  AE.Bridge.callRuby("receive_elevation", json["astergdem"]);
  ready = true;
};



return self;
}(AE.KMLImporter || {}); // end module KMLImporter
