/* module KMLImporterOptions:
 * Methods for the importer of the SketchUp plugin KMLTools.
 */
AE.KMLImporterOptions = function(self) {



var $ = AE.$;



self.initialize = function(options) {

  /* Load defaults.
   * Identifies input elements when their name matches the default option's name.
   */
  AE.Form.fill(options);

  // Loop over all nested inputs and apply event hander for toggleDisabled.
  var inputs = $("input", $("#options"));
  for (var j=0; j<inputs.length; j++) {
    var input = inputs[j];
    if (input.type == "radio" || input.type == "checkbox") {
      input.onclick = function() { toggleDisabled(this); };
      toggleDisabled(input);
    };
  };

  // Ok button.
  $("#button_ok").onclick = function() {
    AE.Bridge.updateOptions(AE.Form.read());
    AE.Dialog.close();
  };

  // Prevent page reloading on buttons in form element.
  $("form")[0].onsubmit = function() { return false };

  // Adjust the size (height) when the dialog has been manually resized.
  // Internet Explorer it requires a little delay to complete resizing because it triggers
  // onresize twice (for height & width). Otherwise we would use incorrect
  // dimensions and the dialog flickers.
  window.onresize = function() { window.setTimeout(AE.Dialog.adjustSize, 0) };

  AE.Dialog.adjustSize();
};



/** Disable or enable input elements in cascaded forms.
 *    Args:
 *      input:
 *      1) A checkbox or radio button that has an rel="c1" attribute pointing at
 *         the id of another input/container that should be disabled/enabled.
 *         <somewhere> <checkbox rel="c1"> </somewhere> ...
 *         <somewhereelse> <container id="c1"> <input> </container> </somewhereelse>
 *      2) A container that has only one checkbox/radio button as direct child element.
 *         All other inputs cascaded in subcontainers of this container will be disabled/enabled.
 *         <container> <checkbox> <subcontainer1> <input> </subcontainer1> </container>
 *     Returns:
 *       nothing
 */
var toggleDisabled = function(input) {
  //
  var toggleChildren = function(input) {
    // Look for rel attribute.
    if (input.getAttribute('rel')) {
      // Get the container.
      var relContainer = $("#" + input.getAttribute('rel'));
      relContainer.disabled = !input.checked;
      // Loop over all children of relContainer and set their disabled status.
      var relInputs = $('input', relContainer);
      for (var i=0; i<relInputs.length; i++) {
        relInputs[i].disabled = !input.checked;
      }
    // Parent element must have only one child input and several sub-containers.
    } else {
      // Get all containers that should be enabled/disabled.
      var c = input.parentNode.childNodes
      // If this input is wrapped in a label, then the parent that we want is outside.
      if (input.parentNode.nodeName == "LABEL") {
        c = input.parentNode.parentNode.childNodes;
      }
      var n = []; // Collects a boolean input (Should only contain one element.)
      // Get the boolean input that decides over the subcontainers' disabled state.
      for (var i=0; i<c.length; i++) {
        // Boolean input (checkbox/radio).
        if (c[i].nodeName == "INPUT" &&
            (c[i].type == "checkbox" || c[i].type=="radio")) {
          n.push(c[i]);
        }
        // Nested boolean input inside a label (checkbox/radio).
        else if (c[i].nodeName == "LABEL") {
          var d = c[i].childNodes;
          for (var j=0; j<d.length; j++) {
            if (d[j].nodeName == "INPUT" &&
                (d[j].type == "checkbox" || d[j].type == "radio")) {
              n.push(d[j]);
              break;
            }
          }
        }
      }
      // Toggle the disabled state for all inputs in all containers according to n[0].
      if (n.length == 1) {
        for (var i=0; i<c.length; i++) {
          if (c[i].childNodes.length == 0) { continue; }
          var ci = c[i].getElementsByTagName('input');
          for (var k=0; k<ci.length; k++) {
            if (!(ci[k] == input)) {
              ci[k].disabled = !input.checked;
            }
          }
        }
      }
    }
  }
  toggleChildren(input);
  // radio: Loop over all connected radio buttons (with same name).
  if (input.type == 'radio') {
    var radios = document.getElementsByName(input.name);
    for (var i=0; i<radios.length; i++) {
      if (radios[i] != input) { toggleChildren(radios[i]); }
    }
  }
};



return self;
}(AE.KMLImporterOptions || {}); // end module KMLImporterOptions
